from PyQt5.Qt import qApp
from PyQt5.QtCore import pyqtSignal
from PyQt5.QtWidgets import QPlainTextEdit
import logging


class QPlainTextEditLogger(logging.Handler):

    def __init__(self, parent):
        logging.Handler.__init__(self)
        self.widget = QPlainTextEditLog(parent)

    def emit(self, record):
        msg = self.format(record)
        self.widget.log_update_signal.emit(msg)
        qApp.processEvents()


class QPlainTextEditLog(QPlainTextEdit):
    log_update_signal = pyqtSignal(str)

    def __init__(self, parent):
        super(QPlainTextEdit, self).__init__(parent)
        self.setReadOnly(True)
        self.setBackgroundVisible(True)



__version__ = "1.1.0"

from deriva.qt.common.async_task import async_execute, Task
from deriva.qt.common.log_widget import QPlainTextEditLogger
from deriva.qt.common.table_widget import TableWidget
from deriva.qt.common.json_editor import JSONEditor

from deriva.qt.auth_agent.ui.auth_window import AuthWindow
from deriva.qt.auth_agent.ui.embedded_auth_window import EmbeddedAuthWindow

from deriva.qt.upload_gui.ui.upload_window import UploadWindow
from deriva.qt.upload_gui.deriva_upload_gui import DerivaUploadGUI



import logging
from PyQt5.QtCore import Qt, QEvent, QMetaObject
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QWidget, QMainWindow, QStatusBar, QVBoxLayout, QMessageBox
from deriva.qt import __version__ as VERSION
from deriva.qt.auth_agent.ui.auth_widget import AuthWidget
from deriva.qt.auth_agent.resources import resources


class EmbeddedAuthWindow(QMainWindow):

    def __init__(self,
                 parent,
                 config,
                 credential_file=None,
                 cookie_persistence=False,
                 authentication_success_callback=None,
                 authentication_failure_callback=None,
                 log_level=logging.INFO):
        super(EmbeddedAuthWindow, self).__init__(parent)
        success_callback = \
            self.successCallback if not authentication_success_callback else authentication_success_callback
        failure_callback = \
            self.failureCallback if not authentication_failure_callback else authentication_failure_callback
        self.ui = EmbeddedAuthWindowUI(self,
                                       config,
                                       credential_file,
                                       cookie_persistence,
                                       success_callback,
                                       failure_callback,
                                       log_level)
        self.cookie_persistence = cookie_persistence
        self.log_level = log_level

    def authenticated(self):
        return self.ui.authWidget.authenticated()

    def login(self):
        self.ui.authWidget.login()

    def logout(self, delete_cookies=False):
        self.ui.authWidget.logout(delete_cookies)

    def successCallback(self, **kwargs):
        host = kwargs.get("host")
        if host:
            self.statusBar().showMessage("Authenticated: %s" % host)
        self.hide()

    def failureCallback(self, **kwargs):
        host = kwargs.get("host")
        message = kwargs.get("message", "Unknown Error")
        if host:
            self.statusBar().showMessage(message)
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Critical)
            msg.setWindowTitle("Error authenticating to host: %s" % host)
            msg.setText(message)
            msg.setStandardButtons(QMessageBox.Close)
            msg.exec_()
        self.hide()

    def changeEvent(self, event):
        if event.type() == QEvent.WindowStateChange:
            if self.windowState() & Qt.WindowMinimized:
                event.ignore()
                self.hide()
                return

        super(EmbeddedAuthWindow, self).changeEvent(event)

    def closeEvent(self, event):
        self.logout()


class EmbeddedAuthWindowUI(object):

    def __init__(self,
                 MainWin,
                 config,
                 credential_file,
                 cookie_persistence,
                 success_callback,
                 failure_callback,
                 log_level):

        # Main Window
        MainWin.setObjectName("EmbeddedAuthWindow")
        MainWin.setWindowIcon(QIcon(":/images/keys.png"))
        MainWin.setWindowTitle(MainWin.tr("DERIVA Authentication Agent %s" % VERSION))
        MainWin.resize(1024, 745)
        self.centralWidget = QWidget(MainWin)
        self.centralWidget.setObjectName("centralWidget")
        MainWin.setCentralWidget(self.centralWidget)
        self.verticalLayout = QVBoxLayout(self.centralWidget)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")
        self.authWidget = AuthWidget(MainWin, config, credential_file, cookie_persistence, log_level)
        self.authWidget.setSuccessCallback(success_callback)
        self.authWidget.setFailureCallback(failure_callback)
        self.authWidget.setObjectName("authWidget")
        self.verticalLayout.addWidget(self.authWidget)

        # Status Bar

        self.statusBar = QStatusBar(MainWin)
        self.statusBar.setToolTip("")
        self.statusBar.setStatusTip("")
        self.statusBar.setObjectName("statusBar")
        MainWin.setStatusBar(self.statusBar)

        # finalize UI setup
        QMetaObject.connectSlotsByName(MainWin)



import logging
import os
import urllib.parse
import webbrowser

from PyQt5.QtCore import Qt, QMetaObject, QThreadPool, pyqtSlot, pyqtSignal
from PyQt5.QtWidgets import qApp, QMainWindow, QWidget, QAction, QSizePolicy, QPushButton, QStyle, QSplitter, QLabel, \
    QToolBar, QStatusBar, QVBoxLayout, QHBoxLayout, QTableWidgetItem, QAbstractItemView, QLineEdit, QFileDialog, \
    QMessageBox
from deriva.core import write_config, stob
from deriva.qt import EmbeddedAuthWindow, QPlainTextEditLogger, Task, TableWidget, __version__
from deriva.qt.upload_gui.impl.upload_tasks import *
from deriva.qt.upload_gui.ui.options_window import OptionsDialog
from deriva.qt.upload_gui.resources import resources


class UploadWindow(QMainWindow):
    progress_update_signal = pyqtSignal(str)

    def __init__(self,
                 uploader,
                 config_file=None,
                 credential_file=None,
                 hostname=None,
                 window_title=None,
                 cookie_persistence=True):
        super(UploadWindow, self).__init__()
        qApp.aboutToQuit.connect(self.quitEvent)

        assert uploader is not None
        self.uploader = None
        self.auth_window = None
        self.identity = None
        self.current_path = None
        self.uploading = False
        self.save_progress_on_cancel = False

        self.ui = UploadWindowUI(self)
        self.ui.title = window_title if window_title else "Deriva Upload Utility %s" % __version__
        self.setWindowTitle(self.ui.title)

        self.config_file = config_file
        self.credential_file = credential_file
        self.cookie_persistence = cookie_persistence

        self.show()
        self.configure(uploader, hostname)

    def configure(self, uploader, hostname):
        # if a hostname has been provided, it overrides whatever default host a given uploader is configured for
        server = None
        if hostname:
            server = dict()
            if hostname.startswith("http"):
                url = urllib.parse.urlparse(hostname)
                server["protocol"] = url.scheme
                server["host"] = url.netloc
            else:
                server["protocol"] = "https"
                server["host"] = hostname

        # instantiate the uploader...
        # if an uploader instance does not have a default host configured, prompt the user to configure one
        if self.uploader:
            del self.uploader
        self.uploader = uploader(self.config_file, self.credential_file, server, dcctx_cid="gui/DerivaUploadGUI")
        if not self.uploader.server:
            if not self.checkValidServer():
                return
            else:
                self.uploader.setServer(server)

        self.setWindowTitle("%s (%s)" % (self.ui.title, self.uploader.server["host"]))

        self.getNewAuthWindow()
        if not self.checkVersion():
            return
        self.getSession()

    def getNewAuthWindow(self):
        if self.auth_window:
            if self.auth_window.authenticated():
                self.on_actionLogout_triggered()
            self.auth_window.destroy()
            del self.auth_window

        self.auth_window = \
            EmbeddedAuthWindow(self,
                               config=self.uploader.server,
                               cookie_persistence=
                               self.uploader.server.get("cookie_persistence", self.cookie_persistence),
                               authentication_success_callback=self.onLoginSuccess,
                               log_level=logging.getLogger().getEffectiveLevel())
        self.ui.actionLogin.setEnabled(True)

    def getSession(self):
        qApp.setOverrideCursor(Qt.WaitCursor)
        logging.debug("Validating session: %s" % self.uploader.server["host"])
        queryTask = SessionQueryTask(self.uploader)
        queryTask.status_update_signal.connect(self.onSessionResult)
        queryTask.query()

    def onLoginSuccess(self, **kwargs):
        self.auth_window.hide()
        self.uploader.setCredentials(kwargs["credential"])
        self.getSession()

    def enableControls(self):
        self.ui.actionUpload.setEnabled(self.canUpload())
        self.ui.actionRescan.setEnabled(self.current_path is not None and self.auth_window.authenticated())
        self.ui.actionCancel.setEnabled(False)
        self.ui.actionOptions.setEnabled(True)
        self.ui.actionLogin.setEnabled(not self.auth_window.authenticated())
        self.ui.actionLogout.setEnabled(self.auth_window.authenticated())
        self.ui.actionExit.setEnabled(True)
        self.ui.browseButton.setEnabled(True)

    def disableControls(self):
        self.ui.actionUpload.setEnabled(False)
        self.ui.actionRescan.setEnabled(False)
        self.ui.actionOptions.setEnabled(False)
        self.ui.actionLogin.setEnabled(False)
        self.ui.actionLogout.setEnabled(False)
        self.ui.actionExit.setEnabled(False)
        self.ui.browseButton.setEnabled(False)

    def closeEvent(self, event=None):
        self.disableControls()
        if self.uploading:
            self.cancelTasks(self.cancelConfirmation())
        if event:
            event.accept()

    def checkValidServer(self):
        self.restoreCursor()
        if self.uploader.server and self.uploader.server.get("host"):
            return True
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setWindowTitle("No Server Configured")
        msg.setText("Add server configuration now?")
        msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        ret = msg.exec_()
        if ret == QMessageBox.Yes:
            self.on_actionOptions_triggered()
        else:
            return False

    def checkAllowSessionCaching(self):
        client_settings = self.uploader.config.get("client_settings")
        if not client_settings:
            return
        allow_session_caching = stob(client_settings.get("allow_session_caching", True))
        cookie_persistence = self.uploader.server.get("cookie_persistence", False)
        if cookie_persistence != allow_session_caching:
            if not allow_session_caching:
                self.uploader.server["cookie_persistence"] = False
                servers = list()
                for server in self.uploader.getServers():
                    if server.get("host", "") != self.uploader.server.get("host"):
                        servers.append(server)
                servers.append(self.uploader.server)
                setServers = getattr(self.uploader, "setServers", None)
                if callable(setServers):
                    setServers(servers)

    def onServerChanged(self, server):
        if server is None or server == self.uploader.server:
            return

        qApp.setOverrideCursor(Qt.WaitCursor)
        self.uploader.setServer(server)
        self.restoreCursor()
        if not self.checkValidServer():
            return
        self.setWindowTitle("%s (%s)" % (self.ui.title, self.uploader.server["host"]))
        self.getNewAuthWindow()
        self.getSession()

    def cancelTasks(self, save_progress):
        qApp.setOverrideCursor(Qt.WaitCursor)
        self.save_progress_on_cancel = save_progress
        self.uploader.cancel()
        Task.shutdown_all()
        self.statusBar().showMessage("Waiting for background tasks to terminate...")

        while True:
            qApp.processEvents()
            if QThreadPool.globalInstance().waitForDone(10):
                break

        self.uploading = False
        self.statusBar().showMessage("All background tasks terminated successfully")
        self.restoreCursor()

    def uploadCallback(self, **kwargs):
        completed = kwargs.get("completed")
        total = kwargs.get("total")
        file_path = kwargs.get("file_path")
        file_name = os.path.basename(file_path) if file_path else ""
        job_info = kwargs.get("job_info", {})
        job_info.update()
        if completed and total:
            file_name = " [%s]" % file_name
            job_info.update({"completed": completed, "total": total, "host": kwargs.get("host")})
            status = "Uploading file%s: %d%% complete" % (file_name, round(((completed / total) % 100) * 100))
            self.uploader.setTransferState(file_path, job_info)
        else:
            summary = kwargs.get("summary", "")
            file_name = "Uploaded file: [%s] " % file_name
            status = file_name  # + summary
        if status:
            self.progress_update_signal.emit(status)

        if self.uploader.cancelled:
            if self.save_progress_on_cancel:
                return -1
            else:
                return False

        return True

    def restoreCursor(self):
        qApp.restoreOverrideCursor()
        qApp.processEvents()

    def statusCallback(self, **kwargs):
        status = kwargs.get("status")
        self.progress_update_signal.emit(status)

    def displayUploads(self, upload_list):
        keys = ["State",
                "Status",
                "File"]
        hidden = ["State"]
        self.ui.uploadList.setRowCount(len(upload_list))
        self.ui.uploadList.setColumnCount(len(keys))

        rows = 0
        for row in upload_list:
            cols = 0
            for key in keys:
                item = QTableWidgetItem()
                value = row.get(key)
                text = str(value) or ""
                item.setText(text)
                item.setToolTip("<span>" + text + "</span>")
                self.ui.uploadList.setItem(rows, cols, item)
                if key in hidden:
                    self.ui.uploadList.hideColumn(cols)
                cols += 1
            rows += 1

        self.ui.uploadList.setHorizontalHeaderLabels(keys)  # add header names
        self.ui.uploadList.horizontalHeader().setDefaultAlignment(Qt.AlignLeft)  # set alignment
        self.ui.uploadList.resizeColumnToContents(0)

    def canUpload(self):
        return (self.ui.uploadList.rowCount() > 0) and self.auth_window.authenticated()

    def checkVersion(self):
        if not self.uploader.isVersionCompatible():
            self.updateStatus("Version incompatibility detected", "Current version: %s, required version: %s" % (
                self.uploader.getVersion(), self.uploader.getVersionCompatibility()))
            self.disableControls()
            self.ui.actionExit.setEnabled(True)
            self.ui.actionOptions.setEnabled(True)
            self.updateConfirmation()
            return False

        self.checkAllowSessionCaching()
        self.resetUI("Ready...")
        return True

    def updateConfirmation(self):
        url = self.uploader.config.get("version_update_url")
        if not url:
            return
        self.restoreCursor()
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setWindowTitle("Version incompatibility detected!")
        msg.setText("Current version: %s\nRequired version: %s\n\nLaunch browser and download required version?"
                    % (self.uploader.getVersion(), self.uploader.getVersionCompatibility()))
        msg.setInformativeText(
            "Selecting \"Yes\" will close the application and launch an external web browser which will take you to a "
            "download page where you can get the required version of this software.")
        msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        ret = msg.exec_()
        if ret == QMessageBox.Yes:
            webbrowser.open_new(url)
            self.deleteLater()

    def updateConfig(self):
        qApp.setOverrideCursor(Qt.WaitCursor)
        configUpdateTask = ConfigUpdateTask(self.uploader)
        configUpdateTask.status_update_signal.connect(self.onUpdateConfigResult)
        configUpdateTask.update_config()

    def scanDirectory(self, reset=False):
        self.uploader.reset()
        scanTask = ScanDirectoryTask(self.uploader)
        scanTask.status_update_signal.connect(self.onScanResult)
        scanTask.scan(self.current_path)

    @pyqtSlot(str)
    def updateProgress(self, status):
        if status:
            self.statusBar().showMessage(status)
        else:
            self.displayUploads(self.uploader.getFileStatusAsArray())

    @pyqtSlot(str, str)
    def updateStatus(self, status, detail=None, success=True):
        msg = status + ((": %s" % detail) if detail else "")
        logging.info(msg) if success else logging.error(msg)
        self.statusBar().showMessage(status)

    @pyqtSlot(str, str)
    def resetUI(self, status, detail=None, success=True):
        self.updateStatus(status, detail, success)
        self.enableControls()

    @pyqtSlot(str)
    def updateLog(self, text):
        self.ui.logTextBrowser.widget.appendPlainText(text)

    @pyqtSlot(bool, str, str, object)
    def onSessionResult(self, success, status, detail, result):
        self.restoreCursor()
        if success:
            self.identity = result["client"]["id"]
            display_name = result["client"]["full_name"]
            self.setWindowTitle("%s (%s - %s)" % (self.ui.title, self.uploader.server["host"], display_name))
            self.ui.actionLogout.setEnabled(True)
            self.ui.actionLogin.setEnabled(False)
            if self.current_path:
                self.ui.actionRescan.setEnabled(True)
                self.ui.actionUpload.setEnabled(True)
            self.updateStatus("Logged in.")
            self.updateConfig()
        else:
            self.updateStatus("Login required.")

    @pyqtSlot(bool, str, str, object)
    def onUpdateConfigResult(self, success, status, detail, result):
        self.restoreCursor()
        if not success:
            self.resetUI(status, detail)
            return
        if not result:
            return
        confirm_updates = stob(self.uploader.server.get("confirm_updates", False))
        if confirm_updates:
            msg = QMessageBox()
            msg.setIcon(QMessageBox.Information)
            msg.setWindowTitle("Updated Configuration Available")
            msg.setText("Apply updated configuration?")
            msg.setInformativeText(
                "Selecting \"Yes\" will apply the latest configuration from the server and overwrite the existing "
                "default configuration file.\n\nSelecting \"No\" will ignore these updates and continue to use the "
                "existing configuration.\n\nYou should always apply the latest configuration changes from the server "
                "unless you understand the risk involved with using a potentially out-of-date configuration.")

            msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
            ret = msg.exec_()
            if ret == QMessageBox.No:
                return

        write_config(self.uploader.getDeployedConfigFilePath(), result)
        self.uploader.initialize(cleanup=False)
        if not self.checkVersion():
            return
        self.on_actionRescan_triggered()

    @pyqtSlot()
    def on_actionBrowse_triggered(self):
        dialog = QFileDialog()
        path = dialog.getExistingDirectory(self,
                                           "Select Directory",
                                           self.current_path,
                                           QFileDialog.ShowDirsOnly)
        if not path:
            return
        self.current_path = path
        self.ui.pathTextBox.setText(os.path.normpath(self.current_path))
        self.scanDirectory()

    @pyqtSlot()
    def on_actionRescan_triggered(self):
        if not self.current_path:
            return

        self.scanDirectory()

    @pyqtSlot(bool, str, str, object)
    def onScanResult(self, success, status, detail, result):
        self.restoreCursor()
        if success:
            self.displayUploads(self.uploader.getFileStatusAsArray())
            self.ui.actionUpload.setEnabled(self.canUpload())
            self.resetUI("Ready...")
            if self.uploading:
                self.on_actionUpload_triggered()
        else:
            self.resetUI(status, detail, success)

    @pyqtSlot()
    def on_actionUpload_triggered(self):
        if not self.uploading:
            if self.uploader.cancelled:
                self.uploading = True
                self.on_actionRescan_triggered()
                return

        self.disableControls()
        self.ui.actionCancel.setEnabled(True)
        self.save_progress_on_cancel = False
        qApp.setOverrideCursor(Qt.WaitCursor)
        self.uploading = True
        self.updateStatus("Uploading...")
        self.progress_update_signal.connect(self.updateProgress)
        uploadTask = UploadFilesTask(self.uploader)
        uploadTask.status_update_signal.connect(self.onUploadResult)
        uploadTask.upload(status_callback=self.statusCallback, file_callback=self.uploadCallback)

    @pyqtSlot(bool, str, str, object)
    def onUploadResult(self, success, status, detail, result):
        self.restoreCursor()
        self.uploading = False
        self.displayUploads(self.uploader.getFileStatusAsArray())
        if success:
            self.resetUI("Ready.")
        else:
            self.resetUI(status, detail, success)

    @pyqtSlot()
    def on_actionCancel_triggered(self):
        self.cancelTasks(self.cancelConfirmation())
        self.restoreCursor()
        self.displayUploads(self.uploader.getFileStatusAsArray())
        self.resetUI("Ready.")

    @pyqtSlot()
    def on_actionLogin_triggered(self):
        if not self.auth_window:
            if self.checkValidServer():
                self.getNewAuthWindow()
            else:
                return
        self.auth_window.show()
        self.auth_window.login()

    @pyqtSlot()
    def on_actionLogout_triggered(self):
        self.setWindowTitle("%s (%s)" % (self.ui.title, self.uploader.server["host"]))
        self.auth_window.logout()
        self.identity = None
        self.ui.actionUpload.setEnabled(False)
        self.ui.actionRescan.setEnabled(False)
        self.ui.actionLogout.setEnabled(False)
        self.ui.actionLogin.setEnabled(True)
        self.updateStatus("Logged out.")

    @pyqtSlot()
    def on_actionOptions_triggered(self):
        OptionsDialog.getOptions(self)

    @pyqtSlot()
    def on_actionHelp_triggered(self):
        pass

    @pyqtSlot()
    def on_actionExit_triggered(self):
        self.closeEvent()
        qApp.quit()

    def quitEvent(self):
        if self.auth_window:
            self.auth_window.logout(self.logoutConfirmation())
        qApp.closeAllWindows()

    def logoutConfirmation(self):
        if self.auth_window and (not self.auth_window.authenticated() or not self.auth_window.cookie_persistence):
            return
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setWindowTitle("Confirm Action")
        msg.setText("Do you wish to completely logout of the system?")
        msg.setInformativeText("Selecting \"Yes\" will clear the login state and invalidate the current user identity."
                               "\n\nSelecting \"No\" will keep your current identity cached, which will allow you to "
                               "log back in without authenticating until your session expires.\n\nNOTE: Select \"Yes\" "
                               "if this is a shared system using a single user account.")
        msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        ret = msg.exec_()
        if ret == QMessageBox.Yes:
            return True
        return False

    def cancelConfirmation(self):
        self.restoreCursor()
        msg = QMessageBox()
        msg.setIcon(QMessageBox.Warning)
        msg.setWindowTitle("Confirm Action")
        msg.setText("Save progress for the current upload?")
        msg.setInformativeText("Selecting \"Yes\" will attempt to resume this transfer from the point where it was "
                               "cancelled.\n\nSelecting \"No\" will require the transfer to be started over from the "
                               "beginning of file.")
        msg.setStandardButtons(QMessageBox.Yes | QMessageBox.No)
        ret = msg.exec_()
        if ret == QMessageBox.Yes:
            return True
        return False


# noinspection PyArgumentList
class UploadWindowUI(object):

    title = "DERIVA File Uploader"

    def __init__(self, MainWin):

        # Main Window
        MainWin.setObjectName("UploadWindow")
        MainWin.setWindowTitle(MainWin.tr(self.title))
        MainWin.resize(1024, 768)
        self.centralWidget = QWidget(MainWin)
        self.centralWidget.setObjectName("centralWidget")
        MainWin.setCentralWidget(self.centralWidget)
        self.verticalLayout = QVBoxLayout(self.centralWidget)
        self.verticalLayout.setContentsMargins(11, 11, 11, 11)
        self.verticalLayout.setSpacing(6)
        self.verticalLayout.setObjectName("verticalLayout")

        self.horizontalLayout = QHBoxLayout()
        self.pathLabel = QLabel("Directory:")
        self.horizontalLayout.addWidget(self.pathLabel)
        self.pathTextBox = QLineEdit()
        self.pathTextBox.setReadOnly(True)
        self.horizontalLayout.addWidget(self.pathTextBox)
        self.browseButton = QPushButton("Browse", self.centralWidget)
        self.browseButton.clicked.connect(MainWin.on_actionBrowse_triggered)
        self.horizontalLayout.addWidget(self.browseButton)
        self.verticalLayout.addLayout(self.horizontalLayout)

        # Splitter for Upload list/Log
        self.splitter = QSplitter(Qt.Vertical)

        # Table View (Upload list)
        self.uploadList = TableWidget(self.centralWidget)
        self.uploadList.setObjectName("uploadList")
        self.uploadList.setStyleSheet(
            """
            QTableWidget {
                    border: 2px solid grey;
                    border-radius: 5px;
            }
            """)
        self.uploadList.setEditTriggers(QAbstractItemView.NoEditTriggers)  # use NoEditTriggers to disable editing
        self.uploadList.setAlternatingRowColors(True)
        self.uploadList.setSelectionBehavior(QAbstractItemView.SelectRows)
        self.uploadList.setSelectionMode(QAbstractItemView.NoSelection)
        self.uploadList.verticalHeader().setDefaultSectionSize(18)  # tighten up the row size
        self.uploadList.horizontalHeader().setStretchLastSection(True)
        self.uploadList.setSortingEnabled(True)  # allow sorting
        self.splitter.addWidget(self.uploadList)

        # Log Widget
        self.logTextBrowser = QPlainTextEditLogger(self.centralWidget)
        self.logTextBrowser.widget.setObjectName("logTextBrowser")
        self.logTextBrowser.widget.setStyleSheet(
            """
            QPlainTextEdit {
                    border: 2px solid grey;
                    border-radius: 5px;
                    background-color: lightgray;
            }
            """)
        self.splitter.addWidget(self.logTextBrowser.widget)

        # add splitter
        self.splitter.setSizes([400, 200])
        self.verticalLayout.addWidget(self.splitter)

    # Actions

        # Browse
        self.actionBrowse = QAction(MainWin)
        self.actionBrowse.setObjectName("actionBrowse")
        self.actionBrowse.setText(MainWin.tr("Browse"))
        self.actionBrowse.setToolTip(MainWin.tr("Set the upload directory"))
        self.actionBrowse.setShortcut(MainWin.tr("Ctrl+B"))

        # Upload
        self.actionUpload = QAction(MainWin)
        self.actionUpload.setObjectName("actionUpload")
        self.actionUpload.setText(MainWin.tr("Upload"))
        self.actionUpload.setToolTip(MainWin.tr("Upload files"))
        self.actionUpload.setShortcut(MainWin.tr("Ctrl+L"))
        self.actionUpload.setEnabled(False)

        # Rescan
        self.actionRescan = QAction(MainWin)
        self.actionRescan.setObjectName("actionRescan")
        self.actionRescan.setText(MainWin.tr("Rescan"))
        self.actionRescan.setToolTip(MainWin.tr("Rescan the upload directory"))
        self.actionRescan.setShortcut(MainWin.tr("Ctrl+R"))
        self.actionRescan.setEnabled(False)

        # Cancel
        self.actionCancel = QAction(MainWin)
        self.actionCancel.setObjectName("actionCancel")
        self.actionCancel.setText(MainWin.tr("Cancel"))
        self.actionCancel.setToolTip(MainWin.tr("Cancel any upload(s) in-progress"))
        self.actionCancel.setShortcut(MainWin.tr("Ctrl+C"))

        # Options
        self.actionOptions = QAction(MainWin)
        self.actionOptions.setObjectName("actionOptions")
        self.actionOptions.setText(MainWin.tr("Options"))
        self.actionOptions.setToolTip(MainWin.tr("Configuration Options"))
        self.actionOptions.setShortcut(MainWin.tr("Ctrl+P"))

        # Login
        self.actionLogin = QAction(MainWin)
        self.actionLogin.setObjectName("actionLogin")
        self.actionLogin.setText(MainWin.tr("Login"))
        self.actionLogin.setToolTip(MainWin.tr("Login to the server"))
        self.actionLogin.setShortcut(MainWin.tr("Ctrl+G"))
        self.actionLogin.setEnabled(False)

        # Logout
        self.actionLogout = QAction(MainWin)
        self.actionLogout.setObjectName("actionLogout")
        self.actionLogout.setText(MainWin.tr("Logout"))
        self.actionLogout.setToolTip(MainWin.tr("Logout of the server"))
        self.actionLogout.setShortcut(MainWin.tr("Ctrl+O"))
        self.actionLogout.setEnabled(False)

        # Exit
        self.actionExit = QAction(MainWin)
        self.actionExit.setObjectName("actionExit")
        self.actionExit.setText(MainWin.tr("Exit"))
        self.actionExit.setToolTip(MainWin.tr("Exit the application"))
        self.actionExit.setShortcut(MainWin.tr("Ctrl+Z"))

        # Help
        self.actionHelp = QAction(MainWin)
        self.actionHelp.setObjectName("actionHelp")
        self.actionHelp.setText(MainWin.tr("Help"))
        self.actionHelp.setToolTip(MainWin.tr("Help"))
        self.actionHelp.setShortcut(MainWin.tr("Ctrl+H"))

    # Menu Bar

        """
        self.menuBar = QMenuBar(MainWin)
        self.menuBar.setObjectName("menuBar")
        MainWin.setMenuBar(self.menuBar)
        self.menuBar.setStyleSheet(
            "QMenuBar{font-family: Arial;font-style: normal;font-size: 10pt;font-weight: bold;};")
        """

    # Tool Bar

        self.mainToolBar = QToolBar(MainWin)
        self.mainToolBar.setObjectName("mainToolBar")
        self.mainToolBar.setToolButtonStyle(Qt.ToolButtonTextUnderIcon)
        self.mainToolBar.setContextMenuPolicy(Qt.PreventContextMenu)
        MainWin.addToolBar(Qt.TopToolBarArea, self.mainToolBar)

        # Upload
        self.mainToolBar.addAction(self.actionUpload)
        self.actionUpload.setIcon(qApp.style().standardIcon(QStyle.SP_FileDialogToParent))

        # Rescan
        self.mainToolBar.addAction(self.actionRescan)
        self.actionRescan.setIcon(qApp.style().standardIcon(QStyle.SP_BrowserReload))

        # Cancel
        self.mainToolBar.addAction(self.actionCancel)
        self.actionCancel.setIcon(qApp.style().standardIcon(QStyle.SP_BrowserStop))
        self.actionCancel.setEnabled(False)

        # Options
        self.mainToolBar.addAction(self.actionOptions)
        self.actionOptions.setIcon(qApp.style().standardIcon(QStyle.SP_FileDialogDetailedView))

        # this spacer right justifies everything that comes after it
        spacer = QWidget()
        spacer.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        self.mainToolBar.addWidget(spacer)

        # Login
        self.mainToolBar.addAction(self.actionLogin)
        self.actionLogin.setIcon(qApp.style().standardIcon(QStyle.SP_DialogApplyButton))

        # Logout
        self.mainToolBar.addAction(self.actionLogout)
        self.actionLogout.setIcon(qApp.style().standardIcon(QStyle.SP_DialogOkButton))

        # Help
        #self.mainToolBar.addAction(self.actionHelp)
        self.actionHelp.setIcon(qApp.style().standardIcon(QStyle.SP_MessageBoxQuestion))

        # Exit
        self.mainToolBar.addAction(self.actionExit)
        self.actionExit.setIcon(qApp.style().standardIcon(QStyle.SP_DialogCancelButton))

    # Status Bar

        self.statusBar = QStatusBar(MainWin)
        self.statusBar.setToolTip("")
        self.statusBar.setStatusTip("")
        self.statusBar.setObjectName("statusBar")
        MainWin.setStatusBar(self.statusBar)

    # configure logging
        self.logTextBrowser.widget.log_update_signal.connect(MainWin.updateLog)
        self.logTextBrowser.setFormatter(logging.Formatter("%(asctime)s - %(levelname)s - %(message)s"))
        logging.getLogger().addHandler(self.logTextBrowser)

    # finalize UI setup
        QMetaObject.connectSlotsByName(MainWin)



import sys
from PyQt5.QtCore import QObject, QRunnable, QThreadPool, pyqtSignal


def async_execute(task):
    QThreadPool.globalInstance().start(task)


class TaskSignal(QObject):
    callback = pyqtSignal(bool, object)

    def __init__(self, callback, parent=None):
        super(TaskSignal, self).__init__(parent)
        self.callback.connect(callback)


class Task(QRunnable):

    INSTANCES = []

    def __init__(self, method, args, callback):
        super(Task, self).__init__()

        self.method = method
        self.args = args
        self.canceled = False
        self.signal = None
        self.callback = callback
        self.setAutoDelete(True)

        Task.INSTANCES.append(self)

    def cancel(self):
        self.canceled = True

    def signal_success(self, result):
        self.signal.callback.emit(True, result)

    def signal_failure(self, result):
        self.signal.callback.emit(False, result)

    def signal_canceled(self):
        self.signal.callback.emit(False, "Task canceled.")

    def run(self):
        try:
            self.signal = TaskSignal(self.callback)
            if self.canceled:
                self.signal_canceled()
                return
            result = self.method(*self.args)
            if self.canceled:
                self.signal_canceled()
                return
            self.signal_success(result)
        except:
            (etype, value, traceback) = sys.exc_info()
            if self.canceled:
                self.signal_canceled()
                return
            self.signal_failure(str(value))
        finally:
            self.cleanup()

    def cleanup(self):
        if self.signal is not None:
            self.signal.deleteLater()
        try:
            Task.INSTANCES.remove(self)
        except ValueError:
            return

    @staticmethod
    def shutdown_all():
        for task in Task.INSTANCES:
            task.cancel()
        Task.INSTANCES = []



from PyQt5.QtCore import QObject, pyqtSignal
from deriva.core import format_exception
from deriva.transfer import DerivaUpload
from deriva.qt import async_execute, Task


class UploadTask(QObject):
    status_update_signal = pyqtSignal(bool, str, str, object)
    progress_update_signal = pyqtSignal(int, int)

    def __init__(self, uploader, parent=None):
        super(UploadTask, self).__init__(parent)
        assert (uploader is not None and isinstance(uploader, DerivaUpload))
        self.uploader = uploader
        self.task = None

    def start(self):
        async_execute(self.task)

    def cancel(self):
        self.task.cancel()

    def set_status(self, success, status, detail, result):
        self.status_update_signal.emit(success, status, detail, result)

    def result_callback(self, success, result):
        self.set_status(success, str(status), "", result)

    def progress_callback(self, current, maximum):
        if self.task.canceled:
            return False

        self.progress_update_signal.emit(current, maximum)
        return True


class SessionQueryTask(UploadTask):
    def __init__(self, parent=None):
        super(SessionQueryTask, self).__init__(parent)

    def result_callback(self, success, result):
        self.set_status(success,
                        "Session query success" if success else "Session query failure",
                        "" if success else format_exception(result),
                        result.json() if success else None)

    def query(self):
        self.task = Task(self.uploader.catalog.get_authn_session, [], self.result_callback)
        self.start()


class ConfigUpdateTask(UploadTask):
    def __init__(self, parent=None):
        super(ConfigUpdateTask, self).__init__(parent)

    def result_callback(self, success, result):
        self.set_status(success,
                        "Configuration update success" if success else "Configuration update failure",
                        "" if success else format_exception(result),
                        result if success else None)

    def update_config(self):
        self.task = Task(self.uploader.getUpdatedConfig, [], self.result_callback)
        self.start()


class ScanDirectoryTask(UploadTask):
    def __init__(self, parent=None):
        super(ScanDirectoryTask, self).__init__(parent)

    def result_callback(self, success, result):
        self.set_status(success,
                        "Directory scan success" if success else "Directory scan failure.",
                        "" if success else format_exception(result),
                        None)

    def scan(self, path):
        self.task = Task(self.uploader.scanDirectory, [path], self.result_callback)
        self.start()


class UploadFilesTask(UploadTask):
    def __init__(self, parent=None):
        super(UploadFilesTask, self).__init__(parent)

    def result_callback(self, success, result):
        self.set_status(success,
                        "File upload success" if success else "File upload failure",
                        "" if success else format_exception(result),
                        None)

    def upload(self, status_callback=None, file_callback=None):
        self.task = Task(self.uploader.uploadFiles, [status_callback, file_callback], self.result_callback)
        self.start()



import os
import re
import logging
from PyQt5.QtCore import Qt, pyqtSlot
from PyQt5.QtWidgets import QDialog, QVBoxLayout, QHBoxLayout, QLabel, QLineEdit, QPushButton, QFileDialog, \
    QGroupBox, QRadioButton, QComboBox, QCheckBox, QMessageBox, QDialogButtonBox, qApp
from deriva.core import stob
from deriva.transfer import GenericUploader
from deriva.qt import JSONEditor


def warningMessageBox(parent, text, detail):
    msg = QMessageBox(parent)
    msg.setIcon(QMessageBox.Warning)
    msg.setWindowTitle("Attention Required")
    msg.setText(text)
    msg.setInformativeText(detail)
    msg.exec_()


def getServerDisplayName(server):
    host = server.get("host", "")
    desc = server.get("desc", "")
    display_name = ("%s" % "https://" + host) if not desc else ("%s: %s" % (desc, "https://" + host))
    return display_name


class OptionsDialog(QDialog):
    def __init__(self, parent):
        super(OptionsDialog, self).__init__(parent)
        self.reconfigure = False
        self.setWindowTitle("Options")
        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        self.setMinimumWidth(600)
        layout = QVBoxLayout(self)
        layout.addStretch(1)

        # Servers
        setServers = getattr(parent.uploader, "setServers", None)
        self.serversConfigurable = True if callable(setServers) else False
        self.serversGroupBox = QGroupBox("Servers:", self)
        self.serverLayout = QHBoxLayout()
        self.serverLabel = QLabel("Server:")
        self.serverLayout.addWidget(self.serverLabel)
        self.serverComboBox = QComboBox()
        self.serverComboBox.setEditable(False)
        self.serverComboBox.setMinimumContentsLength(50)
        self.serverComboBox.currentIndexChanged.connect(self.onServerChanged)
        self.serverLayout.addWidget(self.serverComboBox)
        self.serverLayout.addStretch(1)
        self.addServerButton = QPushButton("Add", parent)
        self.addServerButton.clicked.connect(self.onServerAdd)
        self.addServerButton.setEnabled(self.serversConfigurable)
        self.serverLayout.addWidget(self.addServerButton)
        self.editServerButton = QPushButton("Edit", parent)
        self.editServerButton.clicked.connect(self.onServerEdit)
        self.editServerButton.setEnabled(self.serversConfigurable)
        self.serverLayout.addWidget(self.editServerButton)
        self.removeServerButton = QPushButton("Remove", parent)
        self.removeServerButton.clicked.connect(self.onServerRemove)
        self.removeServerButton.setEnabled(self.serversConfigurable)
        self.serverLayout.addWidget(self.removeServerButton)
        self.serversGroupBox.setLayout(self.serverLayout)
        layout.addWidget(self.serversGroupBox)

        # Configuration
        self.configurable = isinstance(parent.uploader, GenericUploader)
        self.configGroupBox = QGroupBox("Configuration:", self)
        self.configLayout = QHBoxLayout()
        self.configPathLabel = QLabel("File:")
        self.configLayout.addWidget(self.configPathLabel)
        self.configPathTextBox = QLineEdit()
        self.configPathTextBox.setReadOnly(True)
        self.configPathTextBox.setText(os.path.normpath(parent.uploader.getCurrentConfigFilePath()))
        self.configLayout.addWidget(self.configPathTextBox)
        self.configBrowseButton = QPushButton("Change", parent)
        self.configBrowseButton.clicked.connect(self.onConfigChange)
        self.configBrowseButton.setEnabled(self.configurable)
        self.configLayout.addWidget(self.configBrowseButton)
        self.configEditButton = QPushButton("Edit", parent)
        self.configEditButton.clicked.connect(self.onConfigEdit)
        self.configEditButton.setEnabled(self.configurable)
        self.configLayout.addWidget(self.configEditButton)
        self.configGroupBox.setLayout(self.configLayout)
        layout.addWidget(self.configGroupBox)

        # Upload
        # self.uploadGroupBox = QGroupBox("Upload:", self)
        # self.uploadLayout = QHBoxLayout()
        # self.uploadAllButton = QRadioButton("Files and Data")
        # self.uploadAllButton.setChecked(True)
        # self.uploadLayout.addWidget(self.uploadAllButton)
        # self.uploadFilesButton = QRadioButton("Files only")
        # self.uploadLayout.addWidget(self.uploadFilesButton)
        # self.uploadDataButton = QRadioButton("Data only")
        # self.uploadLayout.addWidget(self.uploadDataButton)
        # self.uploadGroupBox.setLayout(self.uploadLayout)
        # layout.addWidget(self.uploadGroupBox)

        # Miscellaneous
        self.miscGroupBox = QGroupBox("Miscellaneous:", self)
        self.miscLayout = QHBoxLayout()
        self.debugCheckBox = QCheckBox("Debug logging")
        self.debugCheckBox.setChecked(True if logging.getLogger().getEffectiveLevel() == logging.DEBUG else False)
        self.miscLayout.addWidget(self.debugCheckBox)
        self.miscGroupBox.setLayout(self.miscLayout)
        layout.addWidget(self.miscGroupBox)

        # Button Box
        self.buttonBox = QDialogButtonBox(parent)
        self.buttonBox.setObjectName("buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel|QDialogButtonBox.Ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        layout.addWidget(self.buttonBox)

        self.populateServers()

    @pyqtSlot()
    def onConfigChange(self):
        uploader = self.parent().uploader
        current_path = os.path.normpath(self.configPathTextBox.text())
        new_path = QFileDialog.getOpenFileName(self,
                                               "Select Configuration File",
                                               current_path,
                                               "Configuration Files (*.json)")
        if new_path[0]:
            new_path = os.path.normpath(new_path[0])
            if new_path != current_path:
                self.configPathTextBox.setText(new_path)
                if new_path != uploader.getDefaultConfigFilePath():
                    uploader.override_config_file = new_path
                else:
                    uploader.override_config_file = None
                self.reconfigure = True
            else:
                self.reconfigure = False

    @pyqtSlot()
    def onConfigEdit(self):
        uploader = self.parent().uploader
        configEditor = JSONEditor(self, self.configPathTextBox.text(), "Configuration Editor")
        configEditor.exec_()
        if configEditor.isModified():
            path = self.configPathTextBox.text()
            uploader.override_config_file = path
            self.reconfigure = True
        del configEditor

    @pyqtSlot()
    def onServerAdd(self):
        server = ServerDialog.configureServer(self.parent(), {})
        if not server:
            return
        hostname = server.get("host")
        index = self.serverComboBox.findText(hostname, Qt.MatchExactly)
        if index > -1:
            warningMessageBox(self.parent(), "Server already exists!",
                              "A server configuration for this hostname already exists. "
                              "Please edit that configuration directly if you wish to make changes to it.")
            return
        else:
            index = self.serverComboBox.count()
            self.serverComboBox.insertItem(index, getServerDisplayName(server), server)

        self.updateDefaultServer(index)
        self.editServerButton.setEnabled(self.serversConfigurable)

    @pyqtSlot()
    def onServerEdit(self):
        index = self.serverComboBox.currentIndex()
        server = self.serverComboBox.itemData(index, Qt.UserRole)
        server = ServerDialog.configureServer(self.parent(), server if server else {})
        if not server:
            return
        self.serverComboBox.setItemData(index, server, Qt.UserRole)
        self.updateDefaultServer(index)

    @pyqtSlot()
    def onServerRemove(self):
        index = self.serverComboBox.currentIndex()
        self.serverComboBox.removeItem(index)
        for x in range(self.serverComboBox.count()):
            current = self.serverComboBox.itemData(x, Qt.UserRole)
            if current["default"] is True:
                self.serverComboBox.setCurrentIndex(x)

    @pyqtSlot()
    def onServerChanged(self):
        uploader = self.parent().uploader
        if not uploader.override_config_file:
            server = self.serverComboBox.currentData(Qt.UserRole)
            if not server:
                return
            config_file = os.path.normpath(os.path.join(
                uploader.getDeployedConfigPath(), server.get('host', ''), uploader.DefaultConfigFileName))
            self.configPathTextBox.setText(config_file)
            if not (os.path.exists(config_file) and os.path.isfile(config_file)):
                self.configEditButton.setEnabled(False)
            elif self.configurable:
                self.configEditButton.setEnabled(True)

    def updateDefaultServer(self, index):
        new = self.serverComboBox.itemData(index, Qt.UserRole)
        if not new.get("default", False):
            return
        else:
            self.serverComboBox.removeItem(index)
            self.serverComboBox.insertItem(0, getServerDisplayName(new), new)
            self.serverComboBox.setCurrentIndex(0)

        for x in range(self.serverComboBox.count()):
            current = self.serverComboBox.itemData(x, Qt.UserRole)
            if new.get("host") != current.get("host"):
                current["default"] = False

    def populateServers(self):
        uploader = self.parent().uploader
        servers = uploader.getServers()
        if not servers:
            self.editServerButton.setEnabled(False)
            return
        else:
            index = 0
            for server in servers:
                self.serverComboBox.insertItem(index, getServerDisplayName(server), server)
                if server == uploader.server:
                    self.serverComboBox.setCurrentIndex(index)
                elif not uploader.server and server.get("default", False):
                    self.serverComboBox.setCurrentIndex(index)
                index += 1
            self.editServerButton.setEnabled(self.serversConfigurable)

    def getServers(self):
        servers = list()
        for x in range(self.serverComboBox.count()):
            servers.append(self.serverComboBox.itemData(x, Qt.UserRole))
        return servers

    @staticmethod
    def getOptions(parent):
        uploader = parent.uploader
        dialog = OptionsDialog(parent)
        ret = dialog.exec_()
        if QDialog.Accepted == ret:
            debug = dialog.debugCheckBox.isChecked()
            logging.getLogger().setLevel(logging.DEBUG if debug else logging.INFO)
            setServers = getattr(uploader, "setServers", None)
            if callable(setServers):
                setServers(dialog.getServers())
            current_server = dialog.serverComboBox.currentData(Qt.UserRole)
            if current_server != uploader.server:
                parent.onServerChanged(current_server)
                return
            if dialog.reconfigure:
                qApp.setOverrideCursor(Qt.WaitCursor)
                uploader.initialize(cleanup=False)
                qApp.restoreOverrideCursor()
                parent.checkVersion()
        del dialog


class ServerDialog(QDialog):
    def __init__(self, parent, server):
        super(ServerDialog, self).__init__(parent)
        self.server = server
        self.setWindowTitle("Server Configuration")
        self.setWindowFlags(self.windowFlags() & ~Qt.WindowContextHelpButtonHint)
        self.setMinimumWidth(400)
        layout = QVBoxLayout(self)

        self.serverLayout = QVBoxLayout(self)
        self.serverGroupBox = QGroupBox("Server:", self)
        self.hostnameLayout = QHBoxLayout()
        self.hostnameLabel = QLabel("Host:")
        self.hostnameLayout.addWidget(self.hostnameLabel)
        self.hostnameTextBox = QLineEdit()
        self.hostnameTextBox.setText(server.get("host", ""))
        self.hostnameLayout.addWidget(self.hostnameTextBox)
        self.serverLayout.addLayout(self.hostnameLayout)

        self.descriptionLayout = QHBoxLayout()
        self.descriptionLabel = QLabel("Description:")
        self.descriptionLayout.addWidget(self.descriptionLabel)
        self.descriptionTextBox = QLineEdit()
        self.descriptionTextBox.setText(server.get("desc", ""))
        self.descriptionLayout.addWidget(self.descriptionTextBox)
        self.serverLayout.addLayout(self.descriptionLayout)

        self.catalogIDLayout = QHBoxLayout()
        self.catalogIDLabel = QLabel("Catalog ID:")
        self.catalogIDLayout.addWidget(self.catalogIDLabel)
        self.catalogIDTextBox = QLineEdit()
        self.catalogIDTextBox.setText(str(server.get("catalog_id", 1)))
        self.catalogIDLayout.addWidget(self.catalogIDTextBox)
        self.serverLayout.addLayout(self.catalogIDLayout)
        self.serverGroupBox.setLayout(self.serverLayout)
        layout.addWidget(self.serverGroupBox)

        setServers = getattr(parent.uploader, "setServers", None)
        self.serversConfigurable = True if callable(setServers) else False
        self.serverOptionsGroupBox = QGroupBox("Options:", self)
        self.checkboxLayout = QHBoxLayout()
        self.defaultServer = QCheckBox("Set as &Default", parent)
        self.defaultServer.setChecked(stob(server.get("default", False)))
        self.defaultServer.setEnabled(self.serversConfigurable)
        self.checkboxLayout.addWidget(self.defaultServer)
        self.confirm_updates = QCheckBox("&Confirm configuration updates", parent)
        self.confirm_updates.setChecked(stob(server.get("confirm_updates", False)))
        self.confirm_updates.setEnabled(self.serversConfigurable)
        self.checkboxLayout.addWidget(self.confirm_updates)

        self.cookie_persistence = QCheckBox("&Stay logged in", parent)
        allow_session_caching = True
        if parent.uploader.config:
            client_settings = parent.uploader.config.get("client_settings")
            if client_settings:
                allow_session_caching = stob(client_settings.get("allow_session_caching", True))
                if not allow_session_caching:
                    server["cookie_persistence"] = False
        self.cookie_persistence.setChecked(stob(server.get("cookie_persistence", False)))
        self.cookie_persistence.setEnabled(self.serversConfigurable and allow_session_caching)
        self.checkboxLayout.addWidget(self.cookie_persistence)
        self.serverOptionsGroupBox.setLayout(self.checkboxLayout)
        layout.addWidget(self.serverOptionsGroupBox)

        # Button Box
        self.buttonBox = QDialogButtonBox(parent)
        self.buttonBox.setObjectName("buttonBox")
        self.buttonBox.setOrientation(Qt.Horizontal)
        self.buttonBox.setStandardButtons(QDialogButtonBox.Cancel | QDialogButtonBox.Ok)
        self.buttonBox.accepted.connect(self.accept)
        self.buttonBox.rejected.connect(self.reject)
        layout.addWidget(self.buttonBox)

    def validate(self):
        host = self.hostnameTextBox.text()
        hostname = re.sub("(?i)^.*https?://", '', host)
        if not hostname:
            warningMessageBox(self.parent(), "Please enter a valid hostname:",
                              "For example: \'www.host.com\' or \'localhost\', etc.")
            return False
        else:
            self.server["host"] = hostname

        self.server["default"] = self.defaultServer.isChecked()
        self.server["confirm_updates"] = self.confirm_updates.isChecked()
        self.server["cookie_persistence"] = self.cookie_persistence.isChecked()

        desc = self.descriptionTextBox.text()
        self.server["desc"] = desc if desc else ""

        catalog_id = int(self.catalogIDTextBox.text())
        self.server["catalog_id"] = catalog_id if catalog_id else 1

        return True

    @staticmethod
    def configureServer(parent, server):
        dialog = ServerDialog(parent, server)
        ret = dialog.exec_()
        if QDialog.Accepted == ret:
            if dialog.validate():
                return dialog.server.copy()
        return None



import sys
import traceback

from PyQt5 import QtCore
from PyQt5.QtGui import QIcon
from PyQt5.QtWidgets import QApplication, QStyleFactory, QMessageBox
from deriva.core import format_exception, BaseCLI
from deriva.transfer import DerivaUpload
from deriva.qt import UploadWindow, __version__


class DerivaUploadGUI(BaseCLI):
    def __init__(self, uploader, description, epilog, cookie_persistence=True, window_icon=":/images/upload.png"):

        if not issubclass(uploader, DerivaUpload):
            raise TypeError("DerivaUpload subclass required")

        BaseCLI.__init__(self, description, epilog, uploader.getVersion())
        self.uploader = uploader
        self.cookie_persistence = cookie_persistence
        self.window_icon = window_icon

    @staticmethod
    def upload_gui(uploader,
                   config_file=None,
                   credential_file=None,
                   hostname=None,
                   window_title=None,
                   window_icon=None,
                   cookie_persistence=True):

        QApplication.setDesktopSettingsAware(False)
        QApplication.setStyle(QStyleFactory.create("Fusion"))
        app = QApplication(sys.argv)
        if window_icon:
            app.setWindowIcon(QIcon(window_icon))
        app.setAttribute(QtCore.Qt.AA_UseHighDpiPixmaps)
        window = UploadWindow(uploader,
                              config_file,
                              credential_file,
                              hostname,
                              window_title=window_title,
                              cookie_persistence=cookie_persistence)
        window.show()
        ret = app.exec_()
        window.destroy()

        return ret

    @staticmethod
    def excepthook(etype, value, tb):
        traceback.print_tb(tb)
        print(format_exception(value), file=sys.stderr)
        msg = QMessageBox()
        msg.setText(str(value))
        msg.setStandardButtons(QMessageBox.Close)
        msg.setWindowTitle("Unhandled Exception: %s" % etype.__name__)
        msg.setIcon(QMessageBox.Critical)
        msg.setDetailedText('\n'.join(traceback.format_exception(etype, value, tb)))
        msg.exec_()

    def main(self):
        sys.excepthook = DerivaUploadGUI.excepthook
        self.parser.add_argument(
            "--no-persistence", action="store_true",
            help="Disable cookie and local storage persistence for QtWebEngine.")
        args = self.parse_cli()
        if self.cookie_persistence:
            self.cookie_persistence = not args.no_persistence

        ret = self.upload_gui(self.uploader,
                              config_file=args.config_file,
                              credential_file=args.credential_file,
                              hostname=args.host,
                              window_title="%s %s" % (self.parser.description, __version__),
                              window_icon=self.window_icon,
                              cookie_persistence=self.cookie_persistence)
        return ret



#
# Copyright 2017 University of Southern California
# Distributed under the GNU GPL 3.0 license. See LICENSE for more info.
#

""" Installation script for deriva.qt
"""

from setuptools import setup, find_packages
import re
import io

__version__ = re.search(
    r'__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
    io.open('deriva/qt/__init__.py', encoding='utf_8_sig').read()
    ).group(1)


def get_readme_contents():
    with io.open('README.md') as readme_file:
        return readme_file.read()


url = 'https://github.com/informatics-isi-edu/deriva-qt'
author = 'USC Information Sciences Institute, Informatics Systems Research Division'
author_email = 'isrd-support@isi.edu'


setup(
    name='deriva-qt',
    description='Graphical User Interface tools for DERIVA built on PyQt5',
    long_description="For further information, visit the project [homepage](%s)." % url,
    long_description_content_type='text/markdown',
    url=url,
    author=author,
    author_email=author_email,
    maintainer=author,
    maintainer_email=author_email,
    version=__version__,
    python_requires='>3.5.2',
    packages=find_packages(),
    package_data={'deriva.qt': ['upload_gui/conf/config.json', '*/resources/images/*']},
    entry_points={
        'console_scripts': [
            'deriva-auth = deriva.qt.auth_agent.__main__:main',
            'deriva-upload = deriva.qt.upload_gui.__main__:main'
        ]
    },
    requires=[
        'deriva',
        'PyQt5'
    ],
    install_requires=[
        'deriva>=0.9.0'
    ],
    extras_require={
        'PyQt5': ["PyQt5==5.11.3"],
        'PyQtWebEngine': ["PyQtWebEngine>=5.12.1"]
    },
    license='GNU GPL 3.0',
    classifiers=[
        'Intended Audience :: Science/Research',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: POSIX',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: Microsoft :: Windows',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7'
    ]
)

